#!/usr/local/bin/perl -wT
use strict;
use CGI;
use DBI;
use Digest::MD5 qw(md5_hex);
use Email::Valid::Loose;
use HTML::Template;
use Jcode;
use MIME::Lite;
use POSIX qw(strftime);
use WebDP::Logger;

my $query = CGI->new;
if ($query->request_method eq 'POST') {        #
    do_post($query);                           #
} else {                                       # #1
    do_get($query);                            #
}                                              #

sub do_post {
    my $query = shift;
    my $email = $query->param('email') || '';
    unless (Email::Valid::Loose->address($email)) {                    #
        WebDP::Logger->logit(info => "$email is invalid");             #
        do_get($query, "Invalid Email: $email");                       # #2
        return;                                                        #
    }                                                                  #

    WebDP::Logger->logit(debug => "email $email is OK");
    my $dbh = DBI->connect('dbi:mysql:webdpdownload', 'user', 'pass', { RaiseError => 1 }) or do {
        WebDP::Logger->logit(alert => "db connection fail: $DBI::errstr");
        die $DBI::errstr;
    };

    my $sth = $dbh->prepare(<<'SQL');
INSERT INTO download (email, url_id, expires) VALUES (?, ?, ?)
SQL
    my $url_id  = generate_id($email); # #3
    my $expires = calc_expires(); # #4
    WebDP::Logger->logit(debug => "url_id: $url_id, expires: $expires");

    eval {
        $sth->execute($email, $url_id, $expires); # #5
        mailsend($email, $url_id, $expires); # #6
    };
    if ($@) {
        if ($@ =~ /Duplicate entry/) {
            WebDP::Logger->logit(info => "Duplicate: $email: $@");
            do_get($query, "Duplicate Entry: $email");
            return;
        }
        WebDP::Logger->logit(alert => "insert fail: $@");
        die $@;
    }

    WebDP::Logger->logit(debug => 'all done: print out thanks page');
    my $template = HTML::Template->new(filename => 'thanks.html');
    $template->param(email => $email);
    print $query->header('text/html'), $template->output;
    $dbh->disconnect;
}

sub do_get {
    my($query, $message) = @_;
    my $template = HTML::Template->new(filename => 'entry.html');
    $template->param(message => $message);
    print $query->header('text/html'), $template->output;
}

sub generate_id {
    my $email = shift;
    return md5_hex(md5_hex($email) . '__secret_string__');
}

sub calc_expires {
    return strftime('%Y-%m-%d %H:%M:%S', localtime(time + 24 * 60 * 60));
}

sub mailsend {
    my($email, $url_id, $expires) = @_;
    my $template = HTML::Template->new(filename => 'entry.txt');
    $template->param(url_id => $url_id, expires => $expires);
    my $msg = MIME::Lite->new(
        From => 'sample@example.com',
        To   => $email,
        Subject => 'URL for download',
        Type => 'text/plain; charset=iso-2022-jp',
        Encoding => '7bit',
        Data => Jcode->new($template->output)->iso_2022_jp,
    );
    $msg->send_by_smtp('localhost');
}